unit tineclient;
{  unit tineclient.pas
   interfaces TINE Client API Calls, allow tine client usage in delphi and lazarus
   Tested on:
      - TINE version    4.1.2 and 4.1.3
      - Delphi version  5
      - Lazarus version 0.9.28.2 beta
   Author          Winfried Koehler
   last changed    2010-02-13
   implementation follows as much as possible http://adweb.desy.de/mcs/tine/client_8c.html

   Compiliation on windows 64bit needed some minor adjustments
   Author          Nico van Schagen
   last changed    2022-08-08
   tested on: lazarus 2.2.2 (windows) and lazarus 2.0.6 (ubuntu)
}

interface

uses
{$IfDef WINDOWS}
   windows, { NOTE: not available on Linux }
{$ENDIF}
{$IF DEFINED(LCLWin32) or DEFINED(LCLWin64)}
   dynlibs,  { NOTE: Lazarus needs unit dynlibs to allow DLL loading on winxx }
{$ENDIF}
   dialogs, forms, tinetypes, tinedefs, Sysutils;



type
  TAttachCallback                      = procedure(linkId:integer; completionCode:integer); cdecl;
  TAttachCallback2                     = procedure(linkId:integer; completionCode:integer; reference:pointer); cdecl;
  TAttachLink                          = function (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; pollingRate:integer; callback: TAttachCallback; mode:integer):integer; cdecl;
  TAttachLinkEx                        = function (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; pollingRate:integer; callback: TAttachCallback; mode:integer; callbackID:Longword):integer; cdecl;
  TAttachLinkEx2                       = function (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; pollingRate:integer; callback: TAttachCallback2; mode:integer; callbackID:Longword; reference:pointer):integer; cdecl;
  { Initiates an asynchronous link.
       devName           full device name (/<Context>/<Server>/<Device>) of the device to contact. For example: "/HERA/BPM/WL167". Note that, if devName
                         references a locally registered equipment module, an internal call to ExecLocalLink() is made. For instance: "/LOCAL/BPMEQM/WL167" sets
                         the context to "LOCAL", and "BPMEQM" must be a locally registered equipment module. In such a case, the call can only be successful when
                         the client and server are running together in the same process).
       devProperty       device property requested, for example "ORBIT.X"
       dout              pointer to the output data set, that is, the data set to be returned from the server to the client. The structure element dout->data should
                         be a pointer to a pre-allocated data buffer, which is to receive the returned data. If dout is a NIL pointer, it is assumed that no data are to be returned.
       din               pointer to the input data set, that is, the data set to be sent to the server from the client. If din is a NIL pointer, it is assumed that
                         no data are to be sent to the server
       access            data access mode. This can be any of the TINE access codes ORed together (CA_READ, CA_WRITE, etc.).
       pollingRate       polling rate in milliseconds as seen at the server. For SINGLE transfers, this value also serves as a timeout parameter.
       callback          address of the user-supplied callback routine to be called when data arrive from the server. The prototype of this routine must be as shown,
                         where the first argument (linkId) gives the id of the link which is calling the callback, and the second argument (completionCode) gives the completion code.
                         For successful data transmission, completionCode will always be 0, otherwise it can be interpreted with GetLastLinkError().
       mode              transfer mode (see discussion on Data Transfer Modes).
       callbackID        supplied by the caller as an identifier to be returned in the callback funtion
       reference         user-supplied pointer which will be returned in the callback routine. This can be a very useful means for de-referencing the callback.
           Returns       A positive link handle if successful, otherwise the negative of a TINE completion code which can be interpreted by a call to GetLastLinkError().  }

  TCloseGlobalLink                     = function (linkId:integer):integer; cdecl;
  { Closes an active globals data link. Note that the parameter passed must be the returned link id when the original call to recvNetGlobal() was made.
       If successful, and the link is the last globals link on a context's multicast group, the application will detach from the group. }

  TCloseLink                           = procedure(linkId:integer); cdecl;
  { Cancels an active data link.
       Use this call to cancel an active data link (alias: CloseLink()). Note that the parameter passed must be the returned link id when the original call to AttachLink() or AttachLinkEx() was made. }

  TCloseNetGlobal                      = function (keyword:PChar):integer; cdecl;
  { Closes an active globals data link according to the globals Keyword.
       Use this call to close an active data link. If the call is successful, and the link is the last globals link on a context's multicast group, the application will detach from the group.
       keyword           the globals keyword used to establish the link. }

  TExecLink                            = function (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint):integer; cdecl;
  TExecLinkEx                          = function (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; timeout:word):integer; cdecl;
  { Executes a synchronous link.
    Synchronous data exchange. ExecLink() does not complete until the data transfer has completed or a timeout has ensued. ExecLink() differs from ExecLinkEx()
    in that the default timeout of 1000 msec is assumed
       devName           full device name (/<Context>/<Server>/<Device>) of the device to contact. For example: "/HERA/BPM/WL167". Note that, if devName references a locally registered equipment module, an internal call to ExecLocalLink() is made. For instance: "/LOCAL/BPMEQM/WL167" sets the context to "LOCAL", and "BPMEQM" must be a locally registered equipment module. In such a case, the call can only be successful when the client and server are running together in the same process).
       devProperty       device property requested, for example "ORBIT.X"
       dout              pointer to the output data set, that is, the data set to be returned from the server to the client. The structure element dout->data should be a pointer to a pre-allocated data buffer, which is to receive the returned data. If dout is a NIL pointer, it is assumed that no data are to be returned.
       din               pointer to the input data set, that is, the data set to be sent to the server from the client. If din is a NIL pointer, it is assumed that no data are to be sent to the server
       access            data access mode. This can be any set of access codes ORed together (CA_READ, CA_WRITE, CA_CONNECT, etc.)
       timeout           timeout in milliseconds. When it is known a priori that a call takes a long time to complete than you can set this value accordingly
           Returns       0 if successful, otherwise a TINE completion code which can be interpreted by a call to GetLastLinkError() }

  TFreeAccessLock                      = procedure(context:PChar; server:PChar); cdecl;
  { Frees an access lock on the server specified.
    A client application can obtain an access lock to the given server provided the caller has WRITE privilileges himself. Once the lock is obtained,
    no other process will be allowed WRITE access regardless of any other access control lists. A lock may be preemptive (LOCK_PREEMTIVE) or persistent (LOCK_PERSISTENT).
    Preemptive locks may be aborted (LOCK_ABORT) by other callers with WRITE privilileges. Persistent locks may not. A Client API Calls an free a lock he has obtained by
    calling this method (nominally equivalent to SetAccessLock() + LOCK_CANCEL.
       context           targeted context of the server
       server            targeted device server
           Returns       a tine return code. }

  TGetAccessLockInformation            = function (context:PChar; server:PChar; callerName:pNAME32; callerIp:pNAME32; timeLeft:pNAME32):integer; cdecl;
  { Acquires access lock information from the server specified.
    A client application can obtain an access lock to the given server provided the caller has WRITE privilileges himself. Once the lock is obtained,
    no other process will be allowed WRITE access regardless of any other access control lists. A lock may be preemptive (LOCK_PREEMTIVE) or persistent (LOCK_PERSISTENT).
    Preemptive locks may be aborted (LOCK_ABORT) by other callers with WRITE privilileges. Persistent locks may not. A Client API Callsan obtain the current access lock
    information via this call. This will provide information as to the caller's user name, network address, and time remaining. If there is no access lock on the server
    specified, then the caller's name will be empty, the network address will be "0.0.0.0" and the time remaining will be "0 seconds remaining".
       context           targeted context of the server
       server            targeted device server
       callerName        NAME32 object to hold the user name of the holder of the access lock.
       callerIp          NAME32 object to hold the network address of the holder of the access lock.
       timeLeft          NAME32 object to hold a string given the seconds remaining on the access lock.
           Returns       a tine return code. }

  TGetAccessLockStatus                 = function (context:PChar; server:PChar):integer; cdecl;
  { Acquires the current access lock status on a client's access lock.
    A client application can obtain an access lock to the given server provided the caller has WRITE privilileges himself. Once the lock is obtained,
    no other process will be allowed WRITE access regardless of any other access control lists. A lock may be preemptive (LOCK_PREEMTIVE) or persistent (LOCK_PERSISTENT).
    Preemptive locks may be aborted (LOCK_ABORT) by other callers with WRITE privilileges. Persistent locks may not. A Client API Callsan of course hold different access
    locks to different servers, and can obtain the current lock status for any of them by using this function.
       context           targeted context of the server
       server            targeted device server
           Returns       a tine return code indicating the status of the access lock. }

  TGetAlwaysRetry                      = function ():integer; cdecl;
  { Gets the current setting of the 'Always Retry' flag.
    Client Links can be instructed to always apply the CA_RETRY flag upon initialization by setting this flag to TRUE. If TRUE all link timeouts will be retried up to
    three times before issuing a link timeout notification. The default value is TRUE.
           Returns       the current value of the retry flag. }

  TGetAutoLinkErrorAlarms              = function ():integer; cdecl;
  { Gets the current setting of the autoLinkErrorAlarms flag.
    A server can set automatic 'link_error' alarms when a link to another server goes down. This call retrieves the current value of this setting, (default = TRUE).
           Returns       the current setting of the autoLinkErrorAlarms flag. }

  TGetClnRecvQueueDepth               = function ():integer; cdecl;
  { Gets the default client-side receive queue depth for all client links.
    Asynchronous data acquisition implies a callback handler which accepts and processes incoming data. Under most circumstances the callback handler should process
    the incoming data quickly and return prior to the next data acquisition (otherwise the data rate is set too high to be useful for the client). However circumstances
    arrive where either bursts of data arrive or the client side is otherwise busy processing an atypical event, etc. In these cases queueing the incoming data can be
    beneficial. You can get the default queue depth for initializing links with this routine.
           Returns       the current setting for the default receive queue depth or a negative TINE return code upon error. }

  TGetCompletionDataSize               = function (linkIndex:integer):integer; cdecl;
  TGetCompletionDataSizeFromCallbackId = function (callbackId:integer):integer; cdecl;
  { Returns the most recent data size of the link index / callback identifier supplied.
       linkIndex         link index for which the completed data size is desired
       callbackId        callback id of the link for which the completed data size is desired
           Returns       the most recent data size of the link index supplied. }

  TGetCompletionDataType               = function (linkIndex:integer):integer; cdecl;
  TGetCompletionDataTypeFromCallbackId = function (callbackId:integer):integer; cdecl;
  { Returns the most recent data type of the link index / callback identifier supplied.
       linkIndex         link index for which the data type is desired
       callbackId        callback id of the link for which the completed data size is desired
           Returns       the most recent data type of the link index / callback identifier supplied. }

  TGetCompletionSource                 = function (linkIndex:integer):SmallInt; cdecl;
  { Returns the error source associated with the input link index.
       linkIndex         link index of the link for which the current status string is desired.
           Returns       the error source associated with the input link index as one of LNK_ERROR_LCL or LNK_ERROR_RMT,
                         or -1 if the link index is invalid. }

  TGetCompletionSourceFromCallbackId   = function (callbackId:integer):SmallInt; cdecl;
  { Returns the error source associated with the callback identifier supplied.
       callbackId        callback id of the link for which the current status string is desired.
           Returns       the error source associated with the callback identifier supplied as one of LNK_ERROR_LCL or LNK_ERROR_RMT,
                         or -1 if the callback identifier is invalid. }

  TGetCompletionStatus                 = function (linkIndex:integer):PChar; cdecl;
  TGetCompletionStatusFromCallbackId   = function (callbackId:integer):PChar; cdecl;
  { Returns the error string associated with the input link index / callback identifier.
       linkIndex         link index of the link for which the current status string is desired.
       callbackId        callback id of the link for which the current status string is desired.
           Returns       the error string associated with the input link index, or NIL if the the link index / callback identifier is invalid. }

  TGetConnectionTable                  = function (tbl:pConTblInfo; tblSize:PInteger):integer; cdecl;
  { Gets the current connection table.
    A client's connections are managed and maintained in a connection table. You can retrieve the current connection table by using this call.
       tbl               a reference to block of memory which will hold the ConTblInfo array giving the current Client API Callsonnection table.
       tblSize           (input) is a pointer to an integer value containing the maximum size (number of entries) the the tbl reference can hold.
                         (output) will contain the size of the current connection table.
           Returns       0 upon success or a TINE return code. }

  TGetConnectionTableCapacity          = function ():integer; cdecl;
  { Gets the maximum number of entries allowed in the connection table.
    A client's connections are managed and maintained in a connection table. The size of this table is pre-allocated at initialization time.
    This allows for fast lookups, since a connection ID is simply an entry into the table. If it is known that the client will need a large number
    of simultaneous links then this value should be set accordingly at initialization time.
           Returns       number of entries. }

  TGetCurrentDataStatus                = function (linkIndex:integer):integer; cdecl;
  TGetCurrentDataStatusFromCallbackId  = function (callbackId:integer):integer; cdecl;
  { Returns the data status code associated with the input link index / callback id.
    If the remote equipment module is called, this is the return code delivered. If the link returns or notifies the caller and the equipment module
    has not been called (e.g. in case of time out) this value will be 'not posted'.
       linkIndex         link index of the link for which the current completion code is desired.
       callbackId        callback id of the link for which the current completion code is desired. }

  TGetCurrentDataTimeStamp               = function (linkIndex:integer):Double; cdecl;
  TGetCurrentDataTimeStampFromCallbackId = function (callbackId:integer):Double; cdecl;
  { Returns the data status code associated with the input link index / callback id.
    If the remote equipment module is called, this is the return code delivered. If the link returns or notifies the caller and the equipment module
    has not been called (e.g. in case of time out) this value will be 'not posted'.
       linkIndex         link index of the link for which the current data timestamp is desired.
       callbackId        callback id of the link for which the current data timestamp is desired.
           Returns       data timestamp associated with input link index / callback id. }

  TGetCurrentLinkStatus                = function (linkIndex:integer):integer; cdecl;
  TGetCurrentLinkStatusFromCallbackId  = function (callbackId:integer):integer; cdecl;
  { Returns the completion code associated with the input link index / callback id.
    If the remote equipment module is called, this is the return code delivered. If the link returns or notifies the caller and the equipment module
    has not been called (e.g. in case of time out) this value will be 'not posted'.
       linkIndex         link index of the link for which the current completion code is desired
       callbackId        callback id of the link for which the current completion code is desired.
           Returns       current completion code associated with input link index / callback id. }

  TGetDataFromCallbackId               = function (callbackId:integer; dout:pDTYPE; status:PWord):integer; cdecl;
  TGetDataFromlinkId                   = function (linkId:integer; dout:pDTYPE; status:PWord):integer; cdecl;
  { Supplies the DTYPE data object and call status for the callback ID / link ID in question.
    If the user-supplied callback identifier is unambiguous, the output data information pertaining the the link can be obtained via this call.
    A reference to a DTYPE object will be filled with the current data parameters, including all time and data stamps, transfer reason, data format, etc.
       linkId            link id of the link for which the current completion code is desired
       callbackId        callback id (see AttachLink()) of the link for which the current completion code is desired.
       dout              (output) is a reference to the DTYPE data object to be filled in.
       status            (output) is a reference to an unsigned short integer to receive the call's return code.
           Returns       0 or a TINE error code. }

  TGetDataStampFromCallbackId          = function (callbackId:integer):integer; cdecl;
  TGetDataStamp                        = function (linkId:integer):integer; cdecl;
  { Gets the data stamp with which the incoming data set has been tagged.
    A server can set an integer data stamp with any transaction which will accompany all data transactions. The caller can retrieve this value with this call.
       linkId            connection table link id (returned from a call to AttachLink()).
                         If -1 is passed, then the currently active value for the data stamp is returned.
       callbackId        callback id (supplied with a call to AttachLinkEx()).
                         If -1 is passed, then the currently active value for the data stamp is returned.
           Returns       data stamp if the call succeeds or -1. }

  TGetGloballinkIdFromCallbackId       = function (callbackId:integer):integer; cdecl;
  TGetGloballinkId                     = function (keyword:PChar):integer; cdecl;
  { retrieves the link ID for a globals link
    A server can set an integer data stamp with any transaction which will accompany all data transactions. The caller can retrieve this value with this call.
       keyword           globals keyword passed as an argument to recvNetGlobal().
       callbackId        The ID supplied as argument is the requested callback ID from the original link registration. This link ID is needed as argument to CloseLink().
           Returns       the link table ID for the globals link or a negative TINE error code. }

  TGetGlobalsHeartbeat                 = function ():integer; cdecl;
  { gets the heartbeat (in seconds) during which time no 'timeout' messages will be sent to a globals dispatch routine (client-side) }

  TGetGlobalsTableCapacity             = function ():integer; cdecl;
  { gets the globals table capacity (client-side) }

  TGetGlobalsTimeStamp                 = function (linkId:integer):Double; cdecl;
  { Returns the timestamp of the globals keyword with the given link ID.
       linkId            globals link index for which the timestamp is desired. }

  TGetLastGlobalDataSize               = function (linkIndex:integer):integer; cdecl;
  { Returns the most recent data size of the global link index supplied.
       linkIndex         global link index for which the data size is desired.
                         If i is -1 the last recorded incoming global data size (not a thread-safe thing to do). }

  TGetLastGlobalDataSizeFromCallbackId = function (callbackId:integer):integer; cdecl;
  { Returns the most recent data size of the global link callback id supplied.
    A server can set an integer data stamp with any transaction which will accompany all data transactions. The caller can retrieve this value with this call.
       callbackId        global link callback id for which the data size is desired.
           Returns       the most recent data size of the global link callback id supplied
    Note:
    This is a best effort to determine the propery entry table for the callback id supplied as an argument. It is the duty of the caller to manage a unique set of callback ids for his application. }

  TGetLastLinkError                    = function(completionCode:Smallint; errstr:PChar):PChar; cdecl;
                                         function GetLastLinkErrorAsString(completionCode:Smallint):string;
  { The error string associated with the input error number.
       completionCode    completion code for which the error string is desired.
                         If completionCode is in fact the last Link Error then the returned error string is used,
                         otherwise the system error string for the completion code in question is returned
       errstr            error string buffer which is to contain the string description of the the completionCode parameter.
                         A pointer to the same string buffer is returned (A null pointer returns "[null error string]").
           Returns       the error string associated with the input error number. }
  type
  TGetLinkName                         = function(linkIndex:integer):PChar; cdecl;
  TGetLinkNameFromCallbackId           = function(callbackId:integer):PChar; cdecl;
  { Returns the full link name associated with the input link index.
       linkIndex         link index of the link for which the link name is desired.
       callbackId        callback id of the link for which the link name is desired.
           Returns       the full link name as "/<Context>/<Server>/<Device>[<Property>]", or "link not found" if the callback id is invalid. }

  TGetLinkQueueDepth                   = function(linkId:integer):integer; cdecl;
  { Gets the client-side receive queue depth for the link in question.
    Asynchronous data acquisition implies a callback handler which accepts and processes incoming data. Under most circumstances the callback
    handler should process the incoming data quickly and return prior to the next data acquisition (otherwise the data rate is set too high to
    be useful for the client). However circumstances arrive where either bursts of data arrive or the client side is otherwise busy processing
    an atypical event, etc. In these cases queueing the incoming data can be beneficial. You can get the current queue depth for a specific link
    using this routine.
       linkId            lconnection table link id (returned from a call to AttachLink()).
           Returns       the current queue depth setting for the link in question. }

  TGetSynchronizationTolerance         = function():Double; cdecl;
  { Gets the tolerance used in deciding whether to apply a timestamp offset or not.
    If a TINE time server is running and supplying a system time stamp, and this server is accepting this to synchronize its local clock when
    applying data time stamps, the tolerance used in applying this offset can be read by this routine.
           Returns       toleranceInSeconds is the maximum time difference allowed before setting the data timestamp offset. }

  TGetSystemDataStamp                  = function(linkId:integer):integer; cdecl;
  TGetSystemDataStampFromCallbackId    = function(callbackId:integer):integer; cdecl;
  { Gets the global system data stamp with which the incoming data set has been tagged.
    A server can set a global data stamp value (e.g. cycle id or run number, etc.) which will accompany all data transactions.
    The caller can retrieve this value with this call.
       linkId            connection table link id (returned from a call to AttachLink()).
                         If -1 is passed, then the currently active value for the data stamp is returned.
       callbackId        callback id (supplied with a call to AttachLinkEx()).
                         If -1 is passed, then the currently active value for the data stamp is returned.
           Returns       the data stamp if the call succeeds or -1. }

  TGetTransferFlag                     = function(linkId:integer):word; cdecl;
  TGetTransferFlagFromCallbackId       = function(callbackId:integer):word; cdecl;
  { Gets the data transfer flag for the given link ID / callback identifier.
    The caller can obtain an examine the data transfer reason associated with the last data transfer for the link ID / callback identifier given.
       linkId            connection table link id (returned from a call to AttachLink()).
                         If -1 is passed, then the current value of the transfer flag is returned.
       callbackId        callback id (supplied with a call to AttachLinkEx()).
                         If -1 is passed, then the current value of the transfer flag is returned.
           Returns       the data transfer flag. }

  TModifyLinkAttributes                = function(linkId:integer; access:Smallint; pollingRate:integer; callback: TAttachCallback; mode:integer; callbackId:Longword):word; cdecl;
  { Allows the caller to assign new link attributes to an active link.
    A TINE client by making this call can re-assign a callback function, access mode or polling rate currently in place for an active link.
       linkId            link identifier for the link in question (the returned identifier when the call to AttachLink() or AttachLinkEx()
                         was originally made
       access            data access mode. This can be any combination of TINE access code 'OR'ed together (e.g. CA_READ|CA_WRITE).
       pollingRate       the polling rate in milliseconds as seen at the server.
                         For SINGLE transfers, this value also serves as a timeout parameter.
       callback          address of the user-supplied callback routine to be called when data arrive from the server.
                         The prototype of this routine must be as shown, where the first argument (id) gives the id of the link
                         which is calling the callback, and the second argument (cc) gives the completion code. For successful data transmission,
                         cc will always be 0, otherwise it can be interpreted with GetLastLinkError().
       mode              the transfer mode (see discussion on Data Transfer Modes).
       callbackID        is supplied by the caller as an identifier to be returned in the callback funtion
           Returns       0 if successful, otherwise a TINE completion code which can be interpreted by a call to GetLastLinkError(). }

  TrecvNetGlobal                       = function(keyword:PChar; dout:pDTYPE; callback: TAttachCallback):integer; cdecl;
  TrecvNetGlobalEx                     = function(keyword:PChar; dout:pDTYPE; callback: TAttachCallback; callbackId:Longword):integer; cdecl;
  TrecvNetGlobalEx2                    = function(keyword:PChar; dout:pDTYPE; callback: TAttachCallback2; callbackId:Longword; reference:pointer):integer; cdecl;
  { Initiates a net globals data link.
    A TINE client requests to receive network global parameters via this routine. Note that the client must reside on a network which itself is
    receiving broadcast data or onto which multicasts are being routed in order for this call to take effect.

    recvNetGlobalEx() differs from recvNetGlobal() in that the caller can supply a callback identify to be returned in the callback function. In lieu of this,
    the callback will return the link identifier.

    recvNetGlobalEx2() differs from recvNetGlobal() and recvNetGlobalEx() in that the caller can supply a callback identify and a reference pointer to be returned
    in the callback function. In lieu of this, the callback will return the link identifier and a null pointer.

       keyword           the name of the network global which is desired. For example: 'HEDCCur'.
       dout              is a pointer to the output data set, that is, the data set to be returned from the server to the client.
                         The structure element dout->data should be a pointer to a pre-allocated data buffer, which is to receive the returned data.
                         If dout is a NIL pointer, it is assumed that no data are to be returned.
       callback          is the address of the user-supplied callback routine to be called when data arrive from the server. The prototype of this routine
                         must be as shown, where the first argument gives the id of the link which is calling the callback, and the second argument
                         gives the completion code. For successful data transmission, completionCode will always be 0, otherwise it can be interpreted with GetLastLinkError().
       callbackID        is supplied by the caller as an identifier to be returned in the callback funtion
       reference         is supplied by the caller as a reference to be returned in the callback function. In this way, the callback can then
                         de-reference the originating source of the callback.
           Returns       A positive link handle if successful, otherwise the negative of a TINE completion code which can be interpreted by a call to GetLastLinkError(). }

  TCycleTriggerCallback                = procedure(cycleNumber:integer; cycleStatus:integer; reference:pointer); cdecl;
  TRegisterCycleTriggerFunction        = function(fcn:TCycleTriggerCallback; eqm:PChar; prpLst:PChar; reference:pointer):integer; cdecl;
  { Registers a cycle trigger callback dispatch function.
    If a CYCLER is running in a server's context, then the server will receive 'Cycle Number' events scheduled by the designated CYCLER server. The cycle number
    will make use of the 'System Data Stamp' to tag all data sets obtained from the server. A server can also register a trigger function dispatch routine (or routines)
    to be called when a 'Cycle Number' event occurs. The dispatch routines will be called prior to setting the 'System Stamp' to the new Cycle Number, which will be set
    following the execution of all dispatch routines. Optionally, the server can provide a property (or list of properties) to be scheduled following the dispatch execution.
    This will ensure that such properties will be called immediately following dispatch execution AND contain the most recent Cycle Number as the 'System Data Stamp'.
       fcn               is a reference to the cycle trigger dispatch routine to be called following the reception of a new Cycle Number.
                         The dispatch routine will be called with the current cycle number, the cycle status (possibly 'link_timeout' if no cycle is received within the
                         assigned globals heartbeat (see SetGlobalsHeartbeat)), and an optional 'reference', which is a void pointer of the caller's choosing (and will
                         be returned to the caller in the dispatch routine).
       eqm               the local equipment module name of the desired central server (e.g. 'BPMEQM')
       prpLst            is the property or properties which are to be 'scheduled' following the execution of the dispatch routine. If more than a single property
                         is to be scheduled, this should be a string containing a comma separated list.
       reference         is a caller supplied void pointer which will be returned to the called in the dispatch routine.
           Returns       0 if successful, otherwise a TINE completion code }

  TSetAccessLock                       = function(context:PChar; server:PChar; lockType:integer; lockDuration:integer):integer; cdecl;
  { Acquires an access lock to the server specified.
    A client application can obtain an access lock to the given server provided the caller has WRITE privileges himself. Once the lock is obtained,
    no other process will be allowed WRITE access regardless of any other access control lists. A lock may be preemptive (LOCK_PREEMTIVE) or persistent
    (LOCK_PERSISTENT). Preemptive locks may be aborted (LOCK_ABORT) by other callers with WRITE privileges. Persistent locks may not.
       context           is the targeted context of the server
       server            is the targeted device server
       lockType          is the desired lock type: one of LOCK_CANCEL, LOCK_PREEPMTIVE, LOCK_PERSISTENT or LOCK_ABORT.
       lockDuration      is the requested duration of the lock in seconds. This applies only to preemptive locks.
                         Persistent locks will ignore this parameter and continue to renew the lock until the caller cancels the lock (graceful) or disappears (maximum 60 second wait).
           Returns       a tine return code. }

  TSetAlwaysRetry                      = procedure(value:integer); cdecl;
                                         procedure SetAlwaysRetry(value: boolean);
  { Sets the 'Always Retry' flag to the value given.
    Client Links can be instructed to always apply the CA_RETRY flag upon initialization by setting this flag to TRUE.
    If TINE_TRUE all link timeouts will be retried up to three times before issuing a link timeout notification. The default value is TRUE.
       value             is the value of the retry flag
    NOTE: TINE uses other boolean ord values than DELPHI. SetAlwaysRetry expects *DELPHI* boolean }
  type
  TSetAutoLinkErrorAlarms              = procedure(value:integer); cdecl;
                                         procedure SetAutoLinkErrorAlarms(value:boolean);
   { Sets the autoLinkErrorAlarms flag.
    A server can set automatic 'link_error' alarms when a link to another server goes down. This call sets the value of this flag, (default = TRUE).
       value             is the desired setting of this flag (default = TRUE)
    NOTE: TINE uses other boolean ord values than DELPHI. SetAutoLinkErrorAlarms expects a *DELPHI* boolean }
  type
  TSetAutoLinkWatchdogs                = procedure(value:integer); cdecl;
                                         procedure SetAutoLinkWatchdogs(value:boolean);
  { Enables/Disables automatic link watchdogs.
    On-Data-Change or Event monitor links can sometimes fail to notice a missing server for up to the system heartbeat of 60 seconds. With this
    flag enabled (the default) all such links to a given server are coupled to a watchdog link which will fire the appropriate link notifications
    when a server goes down. If this feature is not desireable, it can be turned off with this call.
       value             is either TRUE to turn this feature on (the default) or FALSE to turn it off
    NOTE: TINE uses other boolean ord values than DELPHI. SetAutoLinkWatchdogs expects a *DELPHI* boolean }
  type
  TSetClnRecvQueueDepth                = procedure(depth:integer); cdecl;
  { Sets the default client-side receive queue depth for all client links.
    Asynchronous data acquisition implies a callback handler which accepts and processes incoming data. Under most circumstances the callback handler
    should process the incoming data quickly and return prior to the next data acquisition (otherwise the data rate is set too high to be useful for
    the client). However circumstances arrive where either bursts of data arrive or the client side is otherwise busy processing an atypical event, etc.
    In these cases queueing the incoming data can be beneficial. You can set the default queue depth for initializing links with this routine.
       depth             requested queue depth (values less than zero are ignored). A value of zero implies no queue will be maintained. }

  TSetConnectionTableCapacity          = procedure(value:integer); cdecl;
  { Sets the maximum number of entries in the connection table.
    A client's connections are managed and maintained in a connection table. The size of this table is pre-allocated at initialization time. This
    allows for fast lookups, since a connection ID is simply an entry into the table. If it is known that the client will need a large number of
    simultaneous links then this value should be set accordingly at initialization time.
       value             is the requested Client API Callsonnection table size (default: 100) }

  TSetGlobalsHeartbeat                 = procedure(value:integer); cdecl;
  { sets the heartbeat (in seconds) during which time no 'timeout' messages will be sent to a globals dispatch routine (client-side)
       value             is the desired heartbeat in seconds (default: 60 seconds) }

  TSetGlobalsTableCapacity             = procedure(value:integer); cdecl;
  { sets the globals table capacity
    This establishes the maximum size of the (client-side) table which is to receive network globals.
       value             is the desired table size. (default : 50)
    Note: this call must be made prior to the client-side registration for any network global in order to have any effect. }

  TSetLinkQueueDepth                   = procedure(linkId:integer; depth:integer); cdecl;
  { Sets the client-side receive queue depth for the link in question.
    Asynchronous data acquisition implies a callback handler which accepts and processes incoming data. Under most circumstances the callback handler
    should process the incoming data quickly and return prior to the next data acquisition (otherwise the data rate is set too high to be useful for
    the client). However circumstances arrive where either bursts of data arrive or the client side is otherwise busy processing an atypical event, etc.
    In these cases queueing the incoming data can be beneficial. You can set the queue depth for a specific link using this routine.
       linkId            is the connection table link id (returned from a call to AttachLink()).
       depth             is the requested queue depth (values less than zero are ignored). A value of zero implies no queue will be maintained. }

  TSetLinkWatchdogPollingInterval      = function(value:integer):integer; cdecl;
  { Sets the link watchdog polling interval to the value given.
    Data-change or event style monitor links will receive data updates only when the examined data have changed or an event has been declared.
    In addition a systematic heartbeat is received at approximately once per minute per monitor link and a systematic watch dog link per target server
    is established (if enabled) at the minimum of the desired remote polling interval and the value given by this call (default = 1000 msec). Thus a
    link timeout notification will be sent to the link callback with minimum latency. The default link watchog polling interval can be set with this function call.
       value             is the desired link watchdog polling interval in milliseconds. }

  TSetNotificationTolerance            = function(linkId:integer; toleranceAbsolute:Single; tolerancePercent:Single):integer; cdecl;
  { Allows the caller to apply a tolerance pertaining to link notification.
    When a TINE client subscribes to data using CM_DATACHANGE mode, the data is sent from server to client when the data change without regard to any
    applied tolerance (zero tolerance). The client application may nonetheless wish to be notified only when the data have changed outside a given tolerance.
    This can be achieved by calling this routine. The same applies to CM_TIMER mode, where the data are sent server to client according to the schedule. In the
    latter case, all data comparisons are made entirely at the client side. Note that if this feature is desired in CM_TIMER mode, the CM_NODUPLICATES switch
    must be applied to the mode parameter in the call to AttachLink() or AttachLinkEx(). In CM_TIMER mode, it is sufficient that the
    SystemSuppressHeartbeatNofification global switch is set to TRUE to use this feature (otherwise append CM_NODUPLICATES).
       linkId            is the link identifier for the link in question (the returned identifier when the call to AttachLink() or AttachLinkEx() was originally made
       toleranceAbsolute if non-zero is a tolerance in the data units of the in-coming data. It is checked against irrespective of the current data values.
       tolerancePercent  if non-zero is a tolerance expressed as a percent, and refers to the previous data set.
    If both an absolute tolerance and a percent tolerance are used, the absolute value of both contributions are added together to define the total tolerance which should be checked against.
           Returns       0 if successful, otherwise a TINE completion code which can be interpreted by a call to GetLastLinkError(). }

  TSetSynchronizationTolerance         = procedure(toleranceInSeconds:Double); cdecl;
  { Sets the tolerance used in deciding whether to apply a timestamp offset or not.
    If a TINE time server is running and supplying a system time stamp, and this server is accepting this to synchronize its local clock when applying data time
    stamps, the tolerance used in applying this offset can be set by this routine.
       toleranceInSeconds is the maximum time difference allowed before setting the data timestamp offset. (default: 0.5 seconds) }

  TUnregisterCycleTriggerFunction      = function(fcn:TCycleTriggerCallback; reference:pointer):integer; cdecl;
  { Unregisters a previously registered cycle trigger callback dispatch function.
    If a cycle trigger event dispatch routine is no longer required, it can be unregistered using this routine.
       fcn               is a reference to the previously registered cycle trigger dispatch routine which is to be removed.
       reference         is a caller supplied void pointer which will be returned to the called in the dispatch routine.
           Returns       0 if successful, otherwise a TINE completion code }

  TGetFormatSize                       = function(fmt:integer):integer; cdecl;
  { returns the size of simple tine types.
           Returns       > 0 if successful, 0 otherwise }

  TSetSuppressHeartbeatNotification    = procedure(value:integer); cdecl;
  { Determines whether CM_DATACHANGE data links signal incoming data by calling the corresonding callback routine even when the incoming data is a HEARTBEAT notification.
    A client may wish to receive data notification events only when the incoming data have changed or the data link has an error,
    and not receive and respond to heartbeat notifications.  Setting this global variable to TRUE will archieve this aim.
       value             determines whether heartbeat notification will be suppressed or not (default: FALSE). -> see also: SetNotificationTolerance() }

  TSystemPresetMemory                  = function():integer; cdecl;
  { Determines whether data requests preset the calling buffer to zero before issuing the call.
    If a server always returns the amount of data requested, then presetting the calling buffer contents can be entirely superfluous, and in the case
    of large data call, can result in a non-negligible increase in the server CPU load. On the other hand, if a server returns less than the amount of data
    asked for, or the caller asks for an array of fixed string length data, there could be unforseen side-effects
    if the calling buffer has not been zeroed out (default: TRUE). }

  TSystemReset                         = function(level:integer):integer; cdecl;
  { Flushes all entry tables and returns all counters and semaphores to their initial state.
    Useful under some circumstances, for instance following program suspension or interrupt due to CNTL-C, etc.
       level             reserved for future use.
           Returns       0 if successful, otherwise a TINE completion code which can be interpreted by a call to GetLastLinkError(). }

  TSystemCycle                         = function(level:integer):integer; cdecl;
                                         function SystemCycle(value: boolean):boolean;
  { Required call for the TINE engine to function property.
    Should be called within an infinite loop.  Prior to the start of this loop, all other device server registration should occur.
    Clients with asynchronous links must see to it that SystemCycle() is called regularly (unless SystemDelay(-1) is used to block the main process).
    NOTE: SystemCycle does will in general NOT monopolize the CPU as for most Operating systems, blocking sockets are used and the TINE Engine will
          itself block on select() for a duration not greater than the SystemPollingRate.
       chkcmd            used to signal the presence of a local command interpreter.  Developers wishing to bypass the built-in command interpreter
                         in favor of another should pass FALSE for this argument
           Returns       'false' under normal circumstances. 'true' indicates an immediate need to re-enter
                         (i.e. following a complete cycle, there are still incomplete data transfer pending). }
  type
  TSystemDelay                         = procedure(msec:integer); cdecl;
  { Blocks any activity inside a TINE task or equipment modules for the specified amount of time.
    The TINE engine will continue to run and process data which is not related to
       1) the equipment module being blocked if _SystemDelay() is called inside an equipment module or..
       2) the background task if _SystemDelay() is called inside a backgroung task.
    NOTE: The TINE engine is designed to run single-threaded.  You are free to use threads and you can specify multi-threaded backgound tasks if you want the TINE
          engine itself to explicitly use threads.  In such cases, calls to 'sleep()' would  have their intended effect.
          Under single-threaded conditions (i.e. DOS, VMS, or you're avoided threads because they complicate things) you should not use 'sleep()' since
          you will block then entire TINE engine.  Use _SystemDelay() instead if you want to wait at a specific spot in your code until some action has finished. }

  procedure InitializeTineclientDll(path:string);
  procedure UninitializeTineclientDll();
  procedure CheckTineclientDll();


{$IFDEF WINDOWS}
     { WINDOWS: DLL late binding using LoadLibrary, GetProcAddress, FreeLibrary }
     var
     LibHandle: THandle;
     AttachLink                            : TAttachLink;
     AttachLinkEx                          : TAttachLinkEx;
     AttachLinkEx2                         : TAttachLinkEx2;
     CloseGlobalLink                       : TCloseGlobalLink;
     CloseLink                             : TCloseLink;
     CloseNetGlobal                        : TCloseNetGlobal;
     ExecLink                              : TExecLink;
     ExecLinkEx                            : TExecLinkEx;
     FreeAccessLock                        : TFreeAccessLock;
     GetAccessLockInformation              : TGetAccessLockInformation;
     GetAccessLockStatus                   : TGetAccessLockStatus;
     GetLastLinkError                      : TGetLastLinkError;
     GetAlwaysRetry                        : TGetAlwaysRetry;
     GetAutoLinkErrorAlarms                : TGetAutoLinkErrorAlarms;
     GetClnRecvQueueDepth                  : TGetClnRecvQueueDepth;
     GetCompletionDataSize                 : TGetCompletionDataSize;
     GetCompletionDataSizeFromCallbackId   : TGetCompletionDataSizeFromCallbackId;
     GetCompletionDataType                 : TGetCompletionDataType;
     GetCompletionDataTypeFromCallbackId   : TGetCompletionDataTypeFromCallbackId;
     GetCompletionSource                   : TGetCompletionSource;
     GetCompletionSourceFromCallbackId     : TGetCompletionSourceFromCallbackId;
     GetCompletionStatus                   : TGetCompletionStatus;
     GetCompletionStatusFromCallbackId     : TGetCompletionStatusFromCallbackId;
     GetConnectionTable                    : TGetConnectionTable;
     GetConnectionTableCapacity            : TGetConnectionTableCapacity;
     GetCurrentDataStatus                  : TGetCurrentDataStatus;
     GetCurrentDataStatusFromCallbackId    : TGetCurrentDataStatusFromCallbackId;
     GetCurrentDataTimeStamp               : TGetCurrentDataTimeStamp;
     GetCurrentDataTimeStampFromCallbackId : TGetCurrentDataTimeStampFromCallbackId;
     GetCurrentLinkStatus                  : TGetCurrentLinkStatus;
     GetCurrentLinkStatusFromCallbackId    : TGetCurrentLinkStatusFromCallbackId;
     GetDataFromCallbackId                 : TGetDataFromCallbackId;
     GetDataFromlinkId                     : TGetDataFromlinkId;
     GetDataStampFromCallbackId            : TGetDataStampFromCallbackId;
     GetDataStamp                          : TGetDataStamp;
     GetGloballinkIdFromCallbackId         : TGetGloballinkIdFromCallbackId;
     GetGloballinkId                       : TGetGloballinkId;
     GetGlobalsHeartbeat                   : TGetGlobalsHeartbeat;
     GetGlobalsTableCapacity               : TGetGlobalsTableCapacity;
     GetGlobalsTimeStamp                   : TGetGlobalsTimeStamp;
     GetLastGlobalDataSize                 : TGetLastGlobalDataSize;
     GetLastGlobalDataSizeFromCallbackId   : TGetLastGlobalDataSizeFromCallbackId;
     GetLinkName                           : TGetLinkName;
     GetLinkNameFromCallbackId             : TGetLinkNameFromCallbackId;
     GetLinkQueueDepth                     : TGetLinkQueueDepth;
     GetSynchronizationTolerance           : TGetSynchronizationTolerance;
     GetSystemDataStamp                    : TGetSystemDataStamp;
     GetSystemDataStampFromCallbackId      : TGetSystemDataStampFromCallbackId;
     GetTransferFlag                       : TGetTransferFlag;
     GetTransferFlagFromCallbackId         : TGetTransferFlagFromCallbackId;
     ModifyLinkAttributes                  : TModifyLinkAttributes;
     recvNetGlobal                         : TrecvNetGlobal;
     recvNetGlobalEx                       : TrecvNetGlobalEx;
     recvNetGlobalEx2                      : TrecvNetGlobalEx2;
     RegisterCycleTriggerFunction          : TRegisterCycleTriggerFunction;
     SetAccessLock                         : TSetAccessLock;
     SetClnRecvQueueDepth                  : TSetClnRecvQueueDepth;
     SetConnectionTableCapacity            : TSetConnectionTableCapacity;
     SetGlobalsHeartbeat                   : TSetGlobalsHeartbeat;
     SetGlobalsTableCapacity               : TSetGlobalsTableCapacity;
     SetLinkQueueDepth                     : TSetLinkQueueDepth;
     SetLinkWatchdogPollingInterval        : TSetLinkWatchdogPollingInterval;
     SetNotificationTolerance              : TSetNotificationTolerance;
     SetSynchronizationTolerance           : TSetSynchronizationTolerance;
     UnregisterCycleTriggerFunction        : TUnregisterCycleTriggerFunction;
     GetFormatSize                         : TGetFormatSize;
     SetSuppressHeartbeatNotification      : TSetSuppressHeartbeatNotification;
     SystemPresetMemory                    : TSystemPresetMemory;
     SystemReset                           : TSystemReset;
     _SystemCycle                          : TSystemCycle;
     SystemDelay                           : TSystemDelay;
{$ELSE}
     { LINUX, direct linking to libtine.so }
     function  AttachLink                            (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; pollingRate:integer; callback: TAttachCallback; mode:integer):integer; cdecl; external 'tine.so';
     function  AttachLinkEx                          (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; pollingRate:integer; callback: TAttachCallback; mode:integer; callbackID:Longword):integer; cdecl; external 'tine.so';
     function  AttachLinkEx2                         (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; pollingRate:integer; callback: TAttachCallback2; mode:integer; callbackID:Longword; reference:pointer):integer; cdecl; external 'tine.so';
     function  CloseGlobalLink                       (linkId:integer):integer; cdecl; external 'tine.so';
     procedure CloseLink                             (linkId:integer); cdecl; external 'tine.so';
     function  CloseNetGlobal                        (keyword:PChar):integer; cdecl; external 'tine.so';
     function  ExecLink                              (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint):integer; cdecl; external 'tine.so';
     function  ExecLinkEx                            (devName:PChar; devProperty:PChar; dout:pDTYPE; din:pDTYPE; access:Smallint; timeout:word):integer; cdecl; external 'tine.so';
     procedure FreeAccessLock                        (context:PChar; server:PChar); cdecl; external 'tine.so';
     function  GetAccessLockInformation              (context:PChar; server:PChar; callerName:pNAME32; callerIp:pNAME32; timeLeft:pNAME32):integer; cdecl; external 'tine.so';
     function  GetAccessLockStatus                   (context:PChar; server:PChar):integer; cdecl; external 'tine.so';
     function  GetAlwaysRetry                        ():integer; cdecl; external 'tine.so';
     function  GetAutoLinkErrorAlarms                ():integer; cdecl; external 'tine.so';
     function  GetClnRecvQueueDepth                  ():integer; cdecl; external 'tine.so';
     function  GetCompletionDataSize                 (linkIndex:integer):integer; cdecl; external 'tine.so';
     function  GetCompletionDataSizeFromCallbackId   (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetCompletionDataType                 (linkIndex:integer):integer; cdecl; external 'tine.so';
     function  GetCompletionDataTypeFromCallbackId   (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetCompletionSource                   (linkIndex:integer):SmallInt; cdecl; external 'tine.so';
     function  GetCompletionSourceFromCallbackId     (callbackId:integer):SmallInt; cdecl; external 'tine.so';
     function  GetCompletionStatus                   (linkIndex:integer):PChar; cdecl; external 'tine.so';
     function  GetCompletionStatusFromCallbackId     (callbackId:integer):PChar; cdecl; external 'tine.so';
     function  GetConnectionTable                    (tbl:pConTblInfo; tblSize:PInteger):integer; cdecl; external 'tine.so';
     function  GetConnectionTableCapacity            ():integer; cdecl; external 'tine.so';
     function  GetCurrentDataStatus                  (linkIndex:integer):integer; cdecl; external 'tine.so';
     function  GetCurrentDataStatusFromCallbackId    (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetCurrentDataTimeStamp               (linkIndex:integer):Double; cdecl; external 'tine.so';
     function  GetCurrentDataTimeStampFromCallbackId (callbackId:integer):Double; cdecl; external 'tine.so';
     function  GetCurrentLinkStatus                  (linkIndex:integer):integer; cdecl; external 'tine.so';
     function  GetCurrentLinkStatusFromCallbackId    (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetDataFromCallbackId                 (callbackId:integer; dout:pDTYPE; status:PWord):integer; cdecl; external 'tine.so';
     function  GetDataFromlinkId                     (linkId:integer; dout:pDTYPE; status:PWord):integer; cdecl; external 'tine.so';
     function  GetDataStampFromCallbackId            (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetDataStamp                          (linkId:integer):integer; cdecl; external 'tine.so';
     function  GetGloballinkIdFromCallbackId         (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetGloballinkId                       (keyword:PChar):integer; cdecl; external 'tine.so';
     function  GetGlobalsHeartbeat                   ():integer; cdecl; external 'tine.so';
     function  GetGlobalsTableCapacity               ():integer; cdecl; external 'tine.so';
     function  GetGlobalsTimeStamp                   (linkId:integer):Double; cdecl; external 'tine.so';
     function  GetLastGlobalDataSize                 (linkIndex:integer):integer; cdecl; external 'tine.so';
     function  GetLastGlobalDataSizeFromCallbackId   (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetLastLinkError                      (completionCode:Smallint; errstr:PChar):PChar; cdecl; external 'tine.so';
     function  GetLinkName                           (linkIndex:integer):PChar; cdecl; external 'tine.so';
     function  GetLinkNameFromCallbackId             (callbackId:integer):PChar; cdecl; external 'tine.so';
     function  GetLinkQueueDepth                     (linkId:integer):integer; cdecl; external 'tine.so';
     function  GetSynchronizationTolerance           ():Double; cdecl; external 'tine.so';
     function  GetSystemDataStamp                    (linkId:integer):integer; cdecl; external 'tine.so';
     function  GetSystemDataStampFromCallbackId      (callbackId:integer):integer; cdecl; external 'tine.so';
     function  GetTransferFlag                       (linkId:integer):word; cdecl; external 'tine.so';
     function  GetTransferFlagFromCallbackId         (callbackId:integer):word; cdecl; external 'tine.so';
     function  ModifyLinkAttributes                  (linkId:integer; access:Smallint; pollingRate:integer; callback: TAttachCallback; mode:integer; callbackId:Longword):word; cdecl; external 'tine.so';
     function  recvNetGlobal                         (keyword:PChar; dout:pDTYPE; callback: TAttachCallback):integer; cdecl; external 'tine.so';
     function  recvNetGlobalEx                       (keyword:PChar; dout:pDTYPE; callback: TAttachCallback; callbackId:Longword):integer; cdecl; external 'tine.so';
     function  recvNetGlobalEx2                      (keyword:PChar; dout:pDTYPE; callback: TAttachCallback2; callbackId:Longword; reference:pointer):integer; cdecl; external 'tine.so';
     procedure CycleTriggerCallback                  (cycleNumber:integer; cycleStatus:integer; reference:pointer); cdecl; external 'tine.so';
     function  RegisterCycleTriggerFunction          (fcn:TCycleTriggerCallback; eqm:PChar; prpLst:PChar; reference:pointer):integer; cdecl; external 'tine.so';
     function  SetAccessLock                         (context:PChar; server:PChar; lockType:integer; lockDuration:integer):integer; cdecl; external 'tine.so';
     procedure __SetAlwaysRetry                      (value:integer); cdecl; external 'tine.so' name 'SetAlwaysRetry';
     procedure __SetAutoLinkErrorAlarms              (value:integer); cdecl; external 'tine.so' name 'SetAutoLinkErrorAlarms';
     procedure __SetAutoLinkWatchdogs                (value:integer); cdecl; external 'tine.so' name 'SetAutoLinkWatchdogs';
     procedure SetAlwaysRetry                        (value:integer); cdecl; external 'tine.so';
     procedure SetAutoLinkErrorAlarms                (value:integer); cdecl; external 'tine.so';
     procedure SetAutoLinkWatchdogs                  (value:integer); cdecl; external 'tine.so';
     procedure SetClnRecvQueueDepth                  (depth:integer); cdecl; external 'tine.so';
     procedure SetConnectionTableCapaciy             (value:integer); cdecl; external 'tine.so';
     procedure SetGlobalsHeartbeat                   (value:integer); cdecl; external 'tine.so';
     procedure SetGlobalsTableCapacity               (value:integer); cdecl; external 'tine.so';
     procedure SetLinkQueueDepth                     (linkId:integer; depth:integer); cdecl; external 'tine.so';
     function  SetLinkWatchdogPollingInterval        (value:integer):integer; cdecl; external 'tine.so';
     function  SetNotificationTolerance              (linkId:integer; toleranceAbsolute:Single; tolerancePercent:Single):integer; cdecl; external 'tine.so';
     procedure SetSynchronizationTolerance           (toleranceInSeconds:Double); cdecl; external 'tine.so';
     function  UnregisterCycleTriggerFunction        (fcn:TCycleTriggerCallback; reference:pointer):integer; cdecl; external 'tine.so';
     function  GetFormatSize                         (fmt:integer):integer; cdecl; external 'tine.so';
     procedure SetSuppressHeartbeatNotification      (value:integer); cdecl; external 'tine.so';
     function  SystemPresetMemory                    ():integer; cdecl; external 'tine.so';
     function  SystemReset                           (level:integer):integer; cdecl; external 'tine.so';
     function  _SystemCycle                          (level:integer):integer; cdecl; external 'tine.so';
     procedure SystemDelay                           (msec:integer); cdecl; external 'tine.so';
{$ENDIF}

implementation

{$IFDEF WINDOWS}
{ these variables are used internally. }
var  __SetAlwaysRetry                      : TSetAlwaysRetry;
     __SetAutoLinkErrorAlarms              : TSetAutoLinkErrorAlarms;
     __SetAutoLinkWatchdogs                : TSetAutoLinkWatchdogs;
{$ENDIF}

procedure InitializeTineclientDll(path:string);
{$IfDef WINDOWS}
var buf:array[0..255] of char;
begin
  {$IfDef Win32}
  path := path+'\TINE32.DLL'; {e.g. c:\Tine\System32}
  {$ENDIF}
  {$IfDef Win64}
  path := path+'\TINE64.DLL'; {e.g. c:\Tine\System32}
  {$ENDIF}
  if (not FileExists(path)) then
     begin
     Showmessage('ERROR: COULD NOT LOAD DLL: '+path);
     application.terminate;
     exit;
     end;
  StrPCopy(buf, path);
  LibHandle := LoadLibrary(buf);
  if (LibHandle = 0) then
     begin
     ShowMessage('Could not load tinexx.dll - EXITING! ('+path+')');
     application.terminate;
     exit;
     end
  else
     begin
     AttachLink                             := TAttachLink                            (GetProcAddress(LibHandle, 'AttachLink'));
     AttachLinkEx                           := TAttachLinkEx                          (GetProcAddress(LibHandle, 'AttachLinkEx'));
     AttachLinkEx2                          := TAttachLinkEx2                         (GetProcAddress(LibHandle, 'AttachLinkEx2'));
     CloseGlobalLink                        := TCloseGlobalLink                       (GetProcAddress(LibHandle, 'CloseGlobalLink'));
     CloseLink                              := TCloseLink                             (GetProcAddress(LibHandle, 'CloseLink'));
     CloseNetGlobal                         := TCloseNetGlobal                        (GetProcAddress(LibHandle, 'CloseNetGlobal'));
     ExecLink                               := TExecLink                              (GetProcAddress(LibHandle, 'ExecLink'));
     ExecLinkEx                             := TExecLinkEx                            (GetProcAddress(LibHandle, 'ExecLinkEx'));
     FreeAccessLock                         := TFreeAccessLock                        (GetProcAddress(LibHandle, 'FreeAccessLock'));
     GetAccessLockInformation               := TGetAccessLockInformation              (GetProcAddress(LibHandle, 'GetAccessLockInformation'));
     GetAccessLockStatus                    := TGetAccessLockStatus                   (GetProcAddress(LibHandle, 'GetAccessLockStatus'));
     GetLastLinkError                       := TGetLastLinkError                      (GetProcAddress(LibHandle, 'GetLastLinkError'));
     GetAlwaysRetry                         := TGetAlwaysRetry                        (GetProcAddress(LibHandle, 'GetAlwaysRetry'));
     GetAutoLinkErrorAlarms                 := TGetAutoLinkErrorAlarms                (GetProcAddress(LibHandle, 'GetAutoLinkErrorAlarms'));
     GetClnRecvQueueDepth                   := TGetClnRecvQueueDepth                  (GetProcAddress(LibHandle, 'GetClnRecvQueueDepth'));
     GetCompletionDataSize                  := TGetCompletionDataSize                 (GetProcAddress(LibHandle, 'GetCompletionDataSize'));
     GetCompletionDataSizeFromCallbackId    := TGetCompletionDataSizeFromCallbackId   (GetProcAddress(LibHandle, 'GetCompletionDataSizeFromCallbackId'));
     GetCompletionDataType                  := TGetCompletionDataType                 (GetProcAddress(LibHandle, 'GetCompletionDataType'));
     GetCompletionDataTypeFromCallbackId    := TGetCompletionDataTypeFromCallbackId   (GetProcAddress(LibHandle, 'GetCompletionDataTypeFromCallbackId'));
     GetCompletionSource                    := TGetCompletionSource                   (GetProcAddress(LibHandle, 'GetCompletionSource'));
     GetCompletionSourceFromCallbackId      := TGetCompletionSourceFromCallbackId     (GetProcAddress(LibHandle, 'GetCompletionSourceFromCallbackId'));
     GetCompletionStatus                    := TGetCompletionStatus                   (GetProcAddress(LibHandle, 'GetCompletionStatus'));
     GetCompletionStatusFromCallbackId      := TGetCompletionStatusFromCallbackId     (GetProcAddress(LibHandle, 'GetCompletionStatusFromCallbackId'));
     GetConnectionTable                     := TGetConnectionTable                    (GetProcAddress(LibHandle, 'GetConnectionTable'));
     GetConnectionTableCapacity             := TGetConnectionTableCapacity            (GetProcAddress(LibHandle, 'GetConnectionTableCapacity'));
     GetCurrentDataStatus                   := TGetCurrentDataStatus                  (GetProcAddress(LibHandle, 'GetCurrentDataStatus'));
     GetCurrentDataStatusFromCallbackId     := TGetCurrentDataStatusFromCallbackId    (GetProcAddress(LibHandle, 'GetCurrentDataStatusFromCallbackId'));
     GetCurrentDataTimeStamp                := TGetCurrentDataTimeStamp               (GetProcAddress(LibHandle, 'GetCurrentDataTimeStamp'));
     GetCurrentDataTimeStampFromCallbackId  := TGetCurrentDataTimeStampFromCallbackId (GetProcAddress(LibHandle, 'GetCurrentDataTimeStampFromCallbackId'));
     GetCurrentLinkStatus                   := TGetCurrentLinkStatus                  (GetProcAddress(LibHandle, 'GetCurrentLinkStatus'));
     GetCurrentLinkStatusFromCallbackId     := TGetCurrentLinkStatusFromCallbackId    (GetProcAddress(LibHandle, 'GetCurrentLinkStatusFromCallbackId'));
     GetDataFromCallbackId                  := TGetDataFromCallbackId                 (GetProcAddress(LibHandle, 'GetDataFromCallbackId'));
     GetDataFromLinkId                      := TGetDataFromLinkId                     (GetProcAddress(LibHandle, 'GetDataFromLinkId'));
     GetDataStampFromCallbackId             := TGetDataStampFromCallbackId            (GetProcAddress(LibHandle, 'GetDataStampFromCallbackId'));
     GetDataStamp                           := TGetDataStamp                          (GetProcAddress(LibHandle, 'GetDataStamp'));
     GetGlobalLinkIdFromCallbackId          := TGetGlobalLinkIdFromCallbackId         (GetProcAddress(LibHandle, 'GetGlobalLinkIdFromCallbackId'));
     GetGlobalLinkId                        := TGetGlobalLinkId                       (GetProcAddress(LibHandle, 'GetGlobalLinkId'));
     GetGlobalsHeartbeat                    := TGetGlobalsHeartbeat                   (GetProcAddress(LibHandle, 'GetGlobalsHeartbeat'));
     GetGlobalsTableCapacity                := TGetGlobalsTableCapacity               (GetProcAddress(LibHandle, 'GetGlobalsTableCapacity'));
     GetGlobalsTimeStamp                    := TGetGlobalsTimeStamp                   (GetProcAddress(LibHandle, 'GetGlobalsTimeStamp'));
     GetLastGlobalDataSize                  := TGetLastGlobalDataSize                 (GetProcAddress(LibHandle, 'GetLastGlobalDataSize'));
     GetLastGlobalDataSizeFromCallbackId    := TGetLastGlobalDataSizeFromCallbackId   (GetProcAddress(LibHandle, 'GetLastGlobalDataSizeFromCallbackId'));
     GetLinkName                            := TGetLinkName                           (GetProcAddress(LibHandle, 'GetLinkName'));
     GetLinkNameFromCallbackId              := TGetLinkNameFromCallbackId             (GetProcAddress(LibHandle, 'GetLinkNameFromCallbackId'));
     GetLinkQueueDepth                      := TGetLinkQueueDepth                     (GetProcAddress(LibHandle, 'GetLinkQueueDepth'));
     GetSynchronizationTolerance            := TGetSynchronizationTolerance           (GetProcAddress(LibHandle, 'GetSynchronizationTolerance'));
     GetSystemDataStamp                     := TGetSystemDataStamp                    (GetProcAddress(LibHandle, 'GetSystemDataStamp'));
     GetSystemDataStampFromCallbackId       := TGetSystemDataStampFromCallbackId      (GetProcAddress(LibHandle, 'GetSystemDataStampFromCallbackId'));
     GetTransferFlag                        := TGetTransferFlag                       (GetProcAddress(LibHandle, 'GetTransferFlag'));
     GetTransferFlagFromCallbackId          := TGetTransferFlagFromCallbackId         (GetProcAddress(LibHandle, 'GetTransferFlagFromCallbackId'));
     ModifyLinkAttributes                   := TModifyLinkAttributes                  (GetProcAddress(LibHandle, 'ModifyLinkAttributes'));
     recvNetGlobal                          := TrecvNetGlobal                         (GetProcAddress(LibHandle, 'recvNetGlobal'));
     recvNetGlobalEx                        := TrecvNetGlobalEx                       (GetProcAddress(LibHandle, 'recvNetGlobalEx'));
     recvNetGlobalEx2                       := TrecvNetGlobalEx2                      (GetProcAddress(LibHandle, 'recvNetGlobalEx2'));
     RegisterCycleTriggerFunction           := TRegisterCycleTriggerFunction          (GetProcAddress(LibHandle, 'RegisterCycleTriggerFunction'));
     SetAccessLock                          := TSetAccessLock                         (GetProcAddress(LibHandle, 'SetAccessLock'));
     __SetAlwaysRetry                       := TSetAlwaysRetry                        (GetProcAddress(LibHandle, 'SetAlwaysRetry'));
     __SetAutoLinkErrorAlarms               := TSetAutoLinkErrorAlarms                (GetProcAddress(LibHandle, 'SetAutoLinkErrorAlarms'));
     __SetAutoLinkWatchdogs                 := TSetAutoLinkWatchdogs                  (GetProcAddress(LibHandle, 'SetAutoLinkWatchdogs'));
     SetClnRecvQueueDepth                   := TSetClnRecvQueueDepth                  (GetProcAddress(LibHandle, 'SetClnRecvQueueDepth'));
     SetConnectionTableCapacity             := TSetConnectionTableCapacity            (GetProcAddress(LibHandle, 'SetConnectionTableCapacity'));
     SetGlobalsHeartbeat                    := TSetGlobalsHeartbeat                   (GetProcAddress(LibHandle, 'SetGlobalsHeartbeat'));
     SetGlobalsTableCapacity                := TSetGlobalsTableCapacity               (GetProcAddress(LibHandle, 'SetGlobalsTableCapacity'));
     SetLinkQueueDepth                      := TSetLinkQueueDepth                     (GetProcAddress(LibHandle, 'SetLinkQueueDepth'));
     SetLinkWatchdogPollingInterval         := TSetLinkWatchdogPollingInterval        (GetProcAddress(LibHandle, 'SetLinkWatchdogPollingInterval'));
     SetNotificationTolerance               := TSetNotificationTolerance              (GetProcAddress(LibHandle, 'SetNotificationTolerance'));
     SetSynchronizationTolerance            := TSetSynchronizationTolerance           (GetProcAddress(LibHandle, 'SetSynchronizationTolerance'));
     UnregisterCycleTriggerFunction         := TUnregisterCycleTriggerFunction        (GetProcAddress(LibHandle, 'UnregisterCycleTriggerFunction'));
     GetFormatSize                          := TGetFormatSize                         (GetProcAddress(LibHandle, 'GetFormatSize'));
     SetSuppressHeartbeatNotification       := TSetSuppressHeartbeatNotification      (GetProcAddress(LibHandle, 'SetSuppressHeartbeatNotification'));
     SystemPresetMemory                     := TSystemPresetMemory                    (GetProcAddress(LibHandle, 'SystemPresetMemory'));
     SystemReset                            := TSystemReset                           (GetProcAddress(LibHandle, '_SystemReset'));
     _SystemCycle                           := TSystemCycle                           (GetProcAddress(LibHandle, '_SystemCycle'));
     SystemDelay                            := TSystemDelay                           (GetProcAddress(LibHandle, '_SystemDelay'));
     end;
  //enshure, that all functions are really found and initialized.
  CheckTineclientDll();
end;
{$ELSE}
begin
end;
{$ENDIF}


procedure CheckTineclientDll();
begin
{$IfDef WINDOWS}
     { NOTE: you should never read those messages here. They will only occur if dll <-> tineclient.pas mismatch or *typo* during debugging. }
     if @AttachLink                             = NIL then begin ShowMessage('AttachLink                            not initialized!'); application.terminate; end;
     if @AttachLinkEx                           = NIL then begin ShowMessage('AttachLinkEx                          not initialized!'); application.terminate; end;
     if @AttachLinkEx2                          = NIL then begin ShowMessage('AttachLinkEx2                         not initialized!'); application.terminate; end;
     if @CloseGlobalLink                        = NIL then begin ShowMessage('CloseGlobalLink                       not initialized!'); application.terminate; end;
     if @CloseLink                              = NIL then begin ShowMessage('CloseLink                             not initialized!'); application.terminate; end;
     if @CloseNetGlobal                         = NIL then begin ShowMessage('CloseNetGlobal                        not initialized!'); application.terminate; end;
     if @ExecLink                               = NIL then begin ShowMessage('ExecLink                              not initialized!'); application.terminate; end;
     if @ExecLinkEx                             = NIL then begin ShowMessage('ExecLinkEx                            not initialized!'); application.terminate; end;
     if @FreeAccessLock                         = NIL then begin ShowMessage('FreeAccessLock                        not initialized!'); application.terminate; end;
     if @GetAccessLockInformation               = NIL then begin ShowMessage('GetAccessLockInformation              not initialized!'); application.terminate; end;
     if @GetAccessLockStatus                    = NIL then begin ShowMessage('GetAccessLockStatus                   not initialized!'); application.terminate; end;
     if @GetLastLinkError                       = NIL then begin ShowMessage('GetLastLinkError                      not initialized!'); application.terminate; end;
     if @GetAlwaysRetry                         = NIL then begin ShowMessage('GetAlwaysRetry                        not initialized!'); application.terminate; end;
     if @GetAutoLinkErrorAlarms                 = NIL then begin ShowMessage('GetAutoLinkErrorAlarms                not initialized!'); application.terminate; end;
     if @GetClnRecvQueueDepth                   = NIL then begin ShowMessage('GetClnRecvQueueDepth                  not initialized!'); application.terminate; end;
     if @GetCompletionDataSize                  = NIL then begin ShowMessage('GetCompletionDataSize                 not initialized!'); application.terminate; end;
     if @GetCompletionDataSizeFromCallbackId    = NIL then begin ShowMessage('GetCompletionDataSizeFromCallbackId   not initialized!'); application.terminate; end;
     if @GetCompletionDataType                  = NIL then begin ShowMessage('GetCompletionDataType                 not initialized!'); application.terminate; end;
     if @GetCompletionDataTypeFromCallbackId    = NIL then begin ShowMessage('GetCompletionDataTypeFromCallbackId   not initialized!'); application.terminate; end;
     if @GetCompletionSource                    = NIL then begin ShowMessage('GetCompletionSource                   not initialized!'); application.terminate; end;
     if @GetCompletionSourceFromCallbackId      = NIL then begin ShowMessage('GetCompletionSourceFromCallbackId     not initialized!'); application.terminate; end;
     if @GetCompletionStatus                    = NIL then begin ShowMessage('GetCompletionStatus                   not initialized!'); application.terminate; end;
     if @GetCompletionStatusFromCallbackId      = NIL then begin ShowMessage('GetCompletionStatusFromCallbackId     not initialized!'); application.terminate; end;
     if @GetConnectionTable                     = NIL then begin ShowMessage('GetConnectionTable                    not initialized!'); application.terminate; end;
     if @GetConnectionTableCapacity             = NIL then begin ShowMessage('GetConnectionTableCapacity            not initialized!'); application.terminate; end;
     if @GetCurrentDataStatus                   = NIL then begin ShowMessage('GetCurrentDataStatus                  not initialized!'); application.terminate; end;
     if @GetCurrentDataStatusFromCallbackId     = NIL then begin ShowMessage('GetCurrentDataStatusFromCallbackId    not initialized!'); application.terminate; end;
     if @GetCurrentDataTimeStamp                = NIL then begin ShowMessage('GetCurrentDataTimeStamp               not initialized!'); application.terminate; end;
     if @GetCurrentDataTimeStampFromCallbackId  = NIL then begin ShowMessage('GetCurrentDataTimeStampFromCallbackId not initialized!'); application.terminate; end;
     if @GetCurrentLinkStatus                   = NIL then begin ShowMessage('GetCurrentLinkStatus                  not initialized!'); application.terminate; end;
     if @GetCurrentLinkStatusFromCallbackId     = NIL then begin ShowMessage('GetCurrentLinkStatusFromCallbackId    not initialized!'); application.terminate; end;
     if @GetDataFromCallbackId                  = NIL then begin ShowMessage('GetDataFromCallbackId                 not initialized!'); application.terminate; end;
     if @GetDataFromlinkId                      = NIL then begin ShowMessage('GetDataFromlinkId                     not initialized!'); application.terminate; end;
     if @GetDataStampFromCallbackId             = NIL then begin ShowMessage('GetDataStampFromCallbackId            not initialized!'); application.terminate; end;
     if @GetDataStamp                           = NIL then begin ShowMessage('GetDataStamp                          not initialized!'); application.terminate; end;
     if @GetGlobalLinkIdFromCallbackId          = NIL then begin ShowMessage('GetGlobalLinkIdFromCallbackId         not initialized!'); application.terminate; end;
     if @GetGlobalLinkId                        = NIL then begin ShowMessage('GetGlobalLinkId                       not initialized!'); application.terminate; end;
     if @GetGlobalsHeartbeat                    = NIL then begin ShowMessage('GetGlobalsHeartbeat                   not initialized!'); application.terminate; end;
     if @GetGlobalsTableCapacity                = NIL then begin ShowMessage('GetGlobalsTableCapacity               not initialized!'); application.terminate; end;
     if @GetGlobalsTimeStamp                    = NIL then begin ShowMessage('GetGlobalsTimeStamp                   not initialized!'); application.terminate; end;
     if @GetLastGlobalDataSize                  = NIL then begin ShowMessage('GetLastGlobalDataSize                 not initialized!'); application.terminate; end;
     if @GetLastGlobalDataSizeFromCallbackId    = NIL then begin ShowMessage('GetLastGlobalDataSizeFromCallbackId   not initialized!'); application.terminate; end;
     if @GetLinkName                            = NIL then begin ShowMessage('GetLinkName                           not initialized!'); application.terminate; end;
     if @GetLinkNameFromCallbackId              = NIL then begin ShowMessage('GetLinkNameFromCallbackId             not initialized!'); application.terminate; end;
     if @GetLinkQueueDepth                      = NIL then begin ShowMessage('GetLinkQueueDepth                     not initialized!'); application.terminate; end;
     if @GetSynchronizationTolerance            = NIL then begin ShowMessage('GetSynchronizationTolerance           not initialized!'); application.terminate; end;
     if @GetSystemDataStamp                     = NIL then begin ShowMessage('GetSystemDataStamp                    not initialized!'); application.terminate; end;
     if @GetSystemDataStampFromCallbackId       = NIL then begin ShowMessage('GetSystemDataStampFromCallbackId      not initialized!'); application.terminate; end;
     if @GetTransferFlag                        = NIL then begin ShowMessage('GetTransferFlag                       not initialized!'); application.terminate; end;
     if @GetTransferFlagFromCallbackId          = NIL then begin ShowMessage('GetTransferFlagFromCallbackId         not initialized!'); application.terminate; end;
     if @ModifyLinkAttributes                   = NIL then begin ShowMessage('ModifyLinkAttributes                  not initialized!'); application.terminate; end;
     if @recvNetGlobal                          = NIL then begin ShowMessage('recvNetGlobal                         not initialized!'); application.terminate; end;
     if @recvNetGlobalEx                        = NIL then begin ShowMessage('recvNetGlobalEx                       not initialized!'); application.terminate; end;
     if @recvNetGlobalEx2                       = NIL then begin ShowMessage('recvNetGlobalEx2                      not initialized!'); application.terminate; end;
     if @RegisterCycleTriggerFunction           = NIL then begin ShowMessage('RegisterCycleTriggerFunction          not initialized!'); application.terminate; end;
     if @SetAccessLock                          = NIL then begin ShowMessage('SetAccessLock                         not initialized!'); application.terminate; end;
     if @SetAlwaysRetry                       = NIL then begin ShowMessage('SetAlwaysRetry                        not initialized!'); application.terminate; end;
     if @SetAutoLinkErrorAlarms               = NIL then begin ShowMessage('SetAutoLinkErrorAlarms                not initialized!'); application.terminate; end;
     if @SetAutoLinkWatchdogs                 = NIL then begin ShowMessage('SetAutoLinkWatchdogs                  not initialized!'); application.terminate; end;
     if @SetClnRecvQueueDepth                   = NIL then begin ShowMessage('SetClnRecvQueueDepth                  not initialized!'); application.terminate; end;
     if @SetConnectionTableCapacity             = NIL then begin ShowMessage('SetConnectionTableCapacity            not initialized!'); application.terminate; end;
     if @SetGlobalsHeartbeat                    = NIL then begin ShowMessage('SetGlobalsHeartbeat                   not initialized!'); application.terminate; end;
     if @SetGlobalsTableCapacity                = NIL then begin ShowMessage('SetGlobalsTableCapacity               not initialized!'); application.terminate; end;
     if @SetLinkQueueDepth                      = NIL then begin ShowMessage('SetLinkQueueDepth                     not initialized!'); application.terminate; end;
     if @SetLinkWatchdogPollingInterval         = NIL then begin ShowMessage('SetLinkWatchdogPollingInterval        not initialized!'); application.terminate; end;
     if @SetNotificationTolerance               = NIL then begin ShowMessage('SetNotificationTolerance              not initialized!'); application.terminate; end;
     if @SetSynchronizationTolerance            = NIL then begin ShowMessage('SetSynchronizationTolerance           not initialized!'); application.terminate; end;
     if @UnregisterCycleTriggerFunction         = NIL then begin ShowMessage('UnregisterCycleTriggerFunction        not initialized!'); application.terminate; end;
     if @GetFormatSize                          = NIL then begin ShowMessage('GetFormatSize                         not initialized!'); application.terminate; end;
     if @SetSuppressHeartbeatNotification       = NIL then begin ShowMessage('SetSuppressHeartbeatNotification      not initialized!'); application.terminate; end;
     if @SystemPresetMemory                     = NIL then begin ShowMessage('SystemPresetMemory                    not initialized!'); application.terminate; end;
     if @SystemReset                            = NIL then begin ShowMessage('SystemReset                           not initialized!'); application.terminate; end;
     if @_SystemCycle                           = NIL then begin ShowMessage('_SystemCycle                          not initialized!'); application.terminate; end;
     if @SystemDelay                            = NIL then begin ShowMessage('SystemDelay                           not initialized!'); application.terminate; end;
{$ENDIF}
end;

procedure UninitializeTineclientDll();
begin
{$IfDef WINDOWS}
  try
    FreeLibrary(LibHandle);
  finally
  end;
{$ENDIF}
end;

// wrapping tines bool to delphi boolean type.
procedure SetAlwaysRetry(value: boolean);                  begin   if (value = true) then    __SetAlwaysRetry(TINE_TRUE)           else   __SetAlwaysRetry(TINE_FALSE);           end;
procedure SetAutoLinkErrorAlarms(value:boolean);           begin   if (value = true) then    __SetAutoLinkErrorAlarms(TINE_TRUE)   else   __SetAutoLinkErrorAlarms(TINE_FALSE);   end;
procedure SetAutoLinkWatchdogs(value:boolean);             begin   if (value = true) then    __SetAutoLinkWatchdogs(TINE_TRUE)     else   __SetAutoLinkWatchdogs(TINE_FALSE);     end;

function SystemCycle(value: boolean):boolean;
 begin
 if (value = true) then
    result := ( _SystemCycle(TINE_TRUE) <> TINE_FALSE )
 else
    result := ( _SystemCycle(TINE_FALSE) <> TINE_FALSE );
end;

// wrapping C char* to delphi string
function GetLastLinkErrorAsString(completionCode:Smallint):string;
var msgBuf : array[0..STATUS_SIZE-1] of char;
begin
GetLastLinkError(completionCode, @msgBuf);
result := msgBuf;
end;

end.
