unit tinetypes;
{
   TINE types for tineclient.pas
   see http://adweb.desy.de/mcs/tine/annotated.html
   interfaces TINE Client API Calls, allow tine client usage in delphi and lazarus
   Tested on:
      - TINE version    4.1.2 and 4.1.3
      - Delphi version  5
      - Lazarus version 0.9.28.2 beta
   Author          Winfried Koehler
   last changed    2010-02-12
}

interface

uses
{$IfDef WINDOWS}
   windows, { NOTE: not available on Linux }
{$ENDIF}
   tinedefs;

const
 { TINE differs in understanding of 'true'/'false' from delphi. :-(
       TINE:     TRUE == (int) -1;        FALSE == (int) 0;
       Delphi:   TRUE == (boolean) true;  FALSE == (boolean) false
 }
 TINE_TRUE                = -1;
 TINE_FALSE               =  0;

type

 { NOTE: DELPHI doesn't know a 'union' type like DUNION is in C lang, but a Variant Record
         works the same way and is used as a replacement. }
 DUNION = record
  case Integer of
    0: (vptr   : Pointer);                                          //< void pointer to data
    1: (cptr   : Pchar);                                            //< char pointer to data
    2: (strptr : PPchar);                                           //< pointer to string array
    3: (bptr   : Pbyte);                                            //< BYTE pointer to data
    4: (sptr   : PSINT16);                                          //< short (int16) pointer to data
    5: (usptr  : PUINT16);                                          //< unsigned short pointer to data
    6: (lptr   : PSINT32);                                          //< long (int32) pointer to data
    7: (ulptr  : PUINT32);                                          //< unsigned long pointer to data
    8: (llptr  : PSINT64);                                          //< long long (int64) pointer to data
    9: (ullptr : PUINT64);                                          //< unsigned long long pointer to data - NOTE: NO uint64_t SUPPORT IN DELPHI - DELIVERS DATA AS ^sint64_t !!
   10: (fptr   : Pfloat);                                           //< float pointer to data
   11: (dptr   : PDouble);
  end;

 pDUNION = ^DUNION;

 DTYPE = record                                                     //< http://adweb.desy.de/mcs/tine/structDTYPE.html
     dArrayLength : UINT32;                                         //< the size of the data set in terms of the format size specified.
     dFormat      : short;                                          //< the TINE format of the data set
     xferReason   : short;                                          //< the 'reason' (CX_RESPONSE, CX_STALE, etc.) the data have been delivered
     dStamp       : int;                                            //< an optional, settable integer stamp which can be used to tag the data set
     sysStamp     : int;                                            //< the return system applied data stamp (only valid with sychronous calls)
     dTimeStamp   : double;                                         //< is the associated (threadsafe) timestamp of the returned data set, valid only for synchronous calls, as the data object is usually no longer in scope for asynchronous calls.
     dTag         : array[0..TAG_NAME_SIZE-1] of Char;              //< an optional data tag used typically to identify user-defined structures
     data         : DUNION;                                         //< a reference to the data set
     end;
 pDTYPE = ^DTYPE;

 ASPECTRUM = record
     comment         : array[0..79] of char;                        //< optional comment
     tm              : SINT32;                                      //< long integer value interpreted as a time_t
     s_start         : float;                                       //< float value giving the x-axis 'start'
     s_inc           : float;                                       //< float value giving the x-axis increment
     status          : SINT32;                                      //< long integer value giving the status
     spectBufferSize : UINT32;                                      //< the size of the spectrum array buffer.
     spectBuffer     : Pfloat;                                      //< a pointer to the spectrum array
     end;
 pASPECTRUM = ^ASPECTRUM;

 DSPECTRUM = record
     comment         : array[0..79] of char;                        //< optional comment
     tm              : SINT32;                                      //< long integer value interpreted as a time_t
     s_start         : float;                                       //< float value giving the x-axis 'start'
     s_inc           : float;                                       //< float value giving the x-axis increment
     status          : SINT32;                                      //< long integer value giving the status
     d_spect_array   : array[0..SPECTRUM_STD_SIZE-1] of float;      //< a static reference to a spectrum buffer
     end;
 pDSPECTRUM = ^DSPECTRUM;

 in_addr = record
    s_addr           : Longword;                                    //< load with inet_pton()
    end;

 sockaddr_in = record                                               //< IPv4 AF_INET sockets
    sin_family       : short;                                       //< e.g. AF_INET, AF_INET6
    sin_port         : word;                                        //< e.g. htons(3490)
    sin_addr         : in_addr;                                     //< see struct in_addr, below
    sin_zero         : array[0..7] of char;                         //< zero this if you want to
    end;

 ClnInfoStruct = record
     accessMode      : short;                                       //< this client's requested access mode: CM_SINGLE, CM_DATACHANGE, CM_TIMER, etc.
     pollingRate     : SINT32;                                      //< this client's requested polling rate
     starttime       : UINT32;                                      //< this client's start of initial subscription
     counter         : UINT16;                                      //< this client's counts remaining in subscription(s)
     userName        : array [0 .. USERNAME_SIZE-1] of char;        //< this client's user name
     addr            : sockaddr_in;                                 //< this client's ip address
     ncontracts      : short;                                       //< this client's total number of contracts
     tineProtocol    : short;                                       //< this client's tine protocol level
     inetProtocol    : short;                                       //< this client's transport protocol: UDP, TCP, STREAM , etc.
     end;
 pClnInfoStruct = ^ClnInfoStruct;

 CLOG = record
     context         : array[0..31] of char;                        //< the logging context (e.g. the machine context)
     tag             : array[0..31] of char;                        //< a logging tag identifier
     logger          : array[0..31] of char;                        //< the user/server/application submitting the logging entry
     text            : array[0..255] of char;                       //< the text to be logged
     priority        : int;                                         //< the log entry priority (CLOG_PRIORITY_NONE, CLOG_PRIORITY_USEFUL, etc.)
     status          : int;                                         //< the log entry status (CLOG_STATUS_NONE, CLOG_STATUS_INFO, CLOG_STATUS_WARN, etc.)
     timeSent        : double;                                      //< local timestamp of the logging application at the time of submission
     timeLogged      : double;                                      //< local timestamp of the central logger at the time of receipt
     end;
 pCLOG = ^CLOG;

 DADDRESS = record
     i1val           : SINT32;                                      //< the first long integer value
     i2val           : SINT32;                                      //< the second long integer value
     i3val           : SINT32;                                      //< the third long integer value
     i4val           : SINT32;                                      //< the fourth long integer value
     end;
 pDADDRESS = ^DADDRESS;
 
 PbitfieldFormat = ^bitfieldFormat;
 bitfieldFormat = record
     bitmask         : UINT32;                                      //< bitfield mask
     bitshift        : int;                                         //< bitfield shift
     field           : array[0..15] of char;                        //< bitfield field name
     nxt             : PbitfieldFormat;                             //< Pointer to next field in this bitfield
     end;

 pDBITFIELD = ^DBITFIELD;
 DBITFIELD = record
     srvName         : array[0..2*CONTEXT_NAME_SIZE+3] of char;     //< server where the bitfield is registered
     name            : array[0..TAG_NAME_SIZE-1] of char;           //< bit field name tag
     fmt             : int;                                         //< TINE format : CF_BYTE, CF_SHORT, CF_LONG
     bfs             : PbitfieldFormat;                             //< structure definition
     next            : pDBITFIELD;                                  //< Pointer to the next element in a linked list.*/
     end;

 DBLDBL                 = record    d1val:double;         d2val:double;                                                      end;  pDBLDBL       = ^DBLDBL;
 DBLDBLDBL              = record    d1val:double;         d2val:double;         d3val:double;                                end;  pDBLDBLDBL    = ^DBLDBLDBL;
 DBLINT                 = record    dval:double;          ival:SINT32;                                                       end;  pDBLINT       = ^DBLINT;
 DFILTER                = record    f1val:float;          f2val:float;          f3val:float;          ival:SINT32;           end;  pDFILTER      = ^DFILTER;
 DPOINT                 = record    x:float;              y:float;                                                           end;  pDPOINT       = ^DPOINT;
 DWINDOW                = record    t1:SINT32;            t2:SINT32;            i1val:SINT32;         i2val:SINT32;          end;  pDWINDOW      = ^DWINDOW;
 FLTFLT                 = record    f1val:float;          f2val:float;                                                       end;  pFLTFLT       = ^FLTFLT;
 FLTFLTINT              = record    f1val:float;          f2val:float;          ival:SINT32;                                 end;  pFLTFLTINT    = ^FLTFLTINT;
 FLTINT                 = record    fval:float;           ival:SINT32;                                                       end;  pFLTINT       = ^FLTINT;
 FLTINTFLTINT           = record    f1val:float;          i1val:SINT32;         f2val:float;          i2val:SINT32;          end;  pFLTINTFLTINT = ^FLTINTFLTINT;
 FLTINTINT              = record    f1val:float;          i1val:SINT32;         i2val:SINT32;                                end;  pFLTINTINT    = ^FLTINTINT;
 FWINDOW                = record    i1val:SINT32;         i2val:SINT32;         f1val:float;          f2val:float;           end;  pFWINDOW      = ^FWINDOW;
 INTFLTINT              = record    i1val:SINT32;         fval:float;           i2val:SINT32;                                end;  pINTFLTINT    = ^INTFLTINT;
 INTINT                 = record    i1val:SINT32;         i2val:SINT32;                                                      end;  pINTINT       = ^INTINT;
 INTINTINT              = record    i1val:SINT32;         i2val:SINT32;         i3val:SINT32;                                end;  pINTINTINT    = ^INTINTINT;
 NAME8                  = record    name: array[0..7]  of char;                                                              end;  pNAME8        = ^NAME8;
 NAME8I                 = record    name: array[0..7]  of char;                 ival:SINT32;                                 end;  pNAME8I       = ^NAME8I;
 NAME16                 = record    name: array[0..15] of char;                                                              end;  pNAME16       = ^NAME16;
 NAME16DBLDBL           = record    name: array[0..15] of char;                 d1val:double;         d2val:double;          end;  pNAME16DBLDBL = ^NAME16DBLDBL;
 NAME16FI               = record    name: array[0..15] of char;                 fval:float;           ival:SINT32;           end;  pNAME16FI     = ^NAME16FI;
 NAME16I                = record    name: array[0..15] of char;                 ival:SINT32;                                 end;  pNAME16I      = ^NAME16I;
 NAME16II               = record    name: array[0..15] of char;                 i1val:SINT32;         i2val:SINT32;          end;  pNAME16II     = ^NAME16II;
 NAME32                 = record    name: array[0..31] of char;                                                              end;  pNAME32       = ^NAME32;
 NAME32DBLDBL           = record    name: array[0..31] of char;                 d1val:double;         d2val:double;          end;  pNAME32DBLDBL = ^NAME32DBLDBL;
 NAME32I                = record    name: array[0..31] of char;                 ival:SINT32;                                 end;  pNAME32I      = ^NAME32I;
 NAME48                 = record    name: array[0..47] of char;                                                              end;  pNAME48       = ^NAME48;
 NAME48I                = record    name: array[0..47] of char;                 ival:SINT32;                                 end;  pNAME48I      = ^NAME48I;
 NAME64                 = record    name: array[0..63] of char;                                                              end;  pNAME64       = ^NAME64;
 NAME64DBL              = record    name: array[0..63] of char;                 dval:double;                                 end;  pNAME64DBL    = ^NAME64DBL;
 NAME64DBLDBL           = record    name: array[0..63] of char;                 d1val:double;         d2val:double;          end;  pNAME64DBLDBL = ^NAME64DBLDBL;
 NAME64I                = record    name: array[0..63] of char;                 ival:SINT32;                                 end;  pNAME64I      = ^NAME64I;
 NAME64DBLDBLDBL        = record    name: array[0..63] of char;                 d1val:double;         d2val:double;          d3val:double; end;   pNAME64DBLDBLDBL = ^NAME64DBLDBLDBL;

 TImageSourceHeader = record
     baseTag         : UINT32;                                      //< base id
     cameraPortId    : UINT32;                                      //< specific id
     versionTag      : UINT32;                                      //< version
     totalLength     : UINT32;                                      //< total length in bytes (all headers + frame)
     timestampSeconds : SINT32;                                     //< image timestamp (UTC seconds)
     timestampMicroseconds : SINT32;                                //< image timestamp (micro seconds)
     cameraPortName  : array[0..63] of char;                        //< camera port name
     end;
 pTImageSourceHeader = ^TImageSourceHeader;

 TImageFrameHeader = record
     sourceWidth     : SINT32;
     sourceHeight    : SINT32;
     aoiWidth        : SINT32;
     aoiHeight       : SINT32;
     xStart          : SINT32;
     yStart          : SINT32;
     bytesPerPixel   : SINT32;
     effectiveBitsPerPixel : SINT32;
     horizontalBinning     : SINT32;
     verticalBinning : SINT32;
     sourceFormat    : SINT32;
     imageFormat     : SINT32;
     frameNumber     : UINT32;
     eventNumber     : UINT32;
     xScale          : float;
     yScale          : float;
     imageRotation   : float;
     fspare1         : float;
     fspare2         : float;
     fspare3         : float;
     imageFlags      : SINT32;
     ispare1         : SINT32;
     ispare2         : SINT32;
     ispare3         : SINT32;
     appendedFrameSize : SINT32;
     end;
 pTImageFrameHeader = ^TImageFrameHeader;

 DIMAGE = record
     sourceHeader    : TImageSourceHeader;                          //< is the TImageSourceHeader defining the image source and camera parameters
     frameHeader     : TImageFrameHeader;                           //< frameHeader is the TImageFrameHeader giving all of the image frame characteristics
     frameBufferSize : UINT32;                                      //< frameBufferSize if the size in bytes of the frame buffer
     frameBuffer     : Pbyte;                                       //< frameBuffer is a reference to the frame buffer holding the current image.
     end;
 pDIMAGE = ^DIMAGE;

 DUNAME = record
     ival            : SINT32;                                      //< first long integer value
     f1val           : float;                                       //< the first float value
     f2val           : float;                                       //< the second float value
     f3val           : float;                                       //< the third float value
     name            : array[0..15] of char;                        //< the name buffer
 end;
 pDUNAME = ^DUNAME;

 DUSTRING = record
     ival            : SINT32;                                      //< first long integer value
     f1val           : float;                                       //< the first float value
     f2val           : float;                                       //< the second float value
     tm              : SINT32;                                      //< the second long integer value (intpreted frequently as a time_t)
     str             : array[0..79] of char;                        //< the string buffer
 end;
 pDUSTRING = ^DUSTRING;

 hdrSPECTRUM = record
     comment         : array[0..79] of char;                        //< optional comment
     tm              : SINT32;                                      //< long integer value interpreted as a time_t
     s_start         : float;                                       //< float value giving the x-axis 'start'
     s_inc           : float;                                       //< float value giving the x-axis increment
     status          : SINT32;                                      //< long integer value giving the status
 end;
 phdrSPECTRUM = ^hdrSPECTRUM;

 HistorySpecification = record
     pollingRate     : int;                                         //< polling rate in msec
     archiveRate     : int;                                         //< archive rate in sec
     depthShort      : int;                                         //< for short term storage
     depthLong       : int;                                         //< for long term storage
     heartbeat       : int;                                         //< archive heartbeat in sec
     pTolerance      : float;                                       //< percent tolerance
     aTolerance      : float;                                       //< absolute tolerance
     rhsServerName   : PChar;                                       //< Remote Server Name
     rhsPropertyName : PChar;                                       //< Remote Property Name
 end;
 pHistorySpecification = ^HistorySpecification;

 HRS = record
     prpName         : array[0..PROPERTY_NAME_SIZE-1] of char;      //< property name of the record. - NOTE: 'property' is part of delphi's language, therefore here 'prpName'
     device          : array[0..DEVICE_NAME_SIZE-1] of char;        //< device name of the record
     datasiz         : int;                                         //< call size of the record
     datafmt         : int;                                         //< calling format of the record
     pollingRate     : int;                                         //< polling rate in msec
     archiveRate     : int;                                         //< archive rate in msec
     depthShort      : int;                                         //< for short term storage
     depthLong       : int;                                         //< for long term storage
     heartbeat       : int;                                         //< archive heartbeat in sec
     pTolerance      : float;                                       //< percent tolerance
     aTolerance      : float;                                       //< absolute tolerance
 end;
 pHRS = ^HRS;

 PropertyQueryStruct = record
     prpName         : array[0..PROPERTY_NAME_SHORTSIZE-1] of char; //< the property name
     prpDescription  : array[0..31] of char;                        //< the property description
     prpSize         : UINT16;                                      //< the maximum allowed size of an output data set associated with the property
     prpFormat       : BYTE;                                        //< the default output data format accepted for this property
     prpAccess       : BYTE;                                        //< the allowed data access for this property (either CA_READ, CA_WRITE or both)
 end;
 pPropertyQueryStruct = ^PropertyQueryStruct;

 PropertyQueryStructEx = record
     prpName         : array[0..PROPERTY_NAME_SHORTSIZE-1] of char; //< the property name
     prpDescription  : array[0..63] of char;                        //< the property description
     prpRedirection  : array[0..STATUS_SHORTSIZE-1] of char;        //< a redirection string if the property is redirected to another server
     prpTag          : array[0..TAG_NAME_SHORTSIZE-1] of char;      //< the data tag for output data sets (CF_STRUCT, CF_BITFIELD, CF_HISTORY, etc.)
     prpTagIn        : array[0..TAG_NAME_SHORTSIZE-1] of char;      //< the data tag for input data sets (CF_STRUCT, CF_BITFIELD, CF_HISTORY, etc.)
     prpUnits        : array[0..15] of char;                        //< a string containing the natural units of the data returned by the property
     prpMinValue     : float;                                       //< the lower limit of valid data associated with the property */
     prpMaxValue     : float;                                       //< the upper limit of valid data associated with the property
     prpSize         : UINT32;                                      //< the maximum allowed size of an output data set associated with the property
     prpSizeIn       : UINT32;                                      //< the maximum allowed size of an input data set associated with the property
     prpNumOverloads : UINT32;                                      //< the number of overloads associated with this property name
     prpHistoryDepthShort : UINT16;                                 //< the depth of the short term history kept for this property
     prpHistoryDepthLong  : UINT16;                                 //< the depth of the long term history kept for this property
     prpFormat       : BYTE;                                        //< the default output data format accepted for this property
     prpFormatIn     : BYTE;                                        //< is the default input data format accepted for this property
     prpAccess       : BYTE;                                        //< the allowed data access for this property (either CA_READ, CA_WRITE or both)
     prpGraphType    : BYTE;                                        //< is the preferred graph type for output data associated with this property, e.g. GT_LIN, GT_LOG, etc
 end;
 pPropertyQueryStructEx = ^PropertyQueryStructEx;

 PrpQueryStruct = record
     prpName         : array[0..PROPERTY_NAME_SIZE-1] of char;      //< the property name
     prpDescription  : array[0..PROPERTY_DESC_SIZE-1] of char;      //< the property description
     prpRedirection  : array[0..PROPERTY_REDIR_SIZE-1] of char;     //< a redirection string if the property is redirected to another server
     prpTag          : array[0..TAG_NAME_SIZE-1] of char;           //< the data tag for output data sets (CF_STRUCT, CF_BITFIELD, CF_HISTORY, etc.)
     prpTagIn        : array[0..TAG_NAME_SIZE-1] of char;           //< the data tag for input data sets (CF_STRUCT, CF_BITFIELD, CF_HISTORY, etc.)
     prpUnits        : array[0..UNITS_SIZE-1] of char;              //< a string containing the natural units of the data returned by the property
     prpMinValue     : float;                                       //< the lower limit of valid data associated with the property
     prpMaxValue     : float;                                       //< the upper limit of valid data associated with the property
     prpSize         : UINT32;                                      //< the maximum allowed size of an output data set associated with the property
     prpSizeIn       : UINT32;                                      //< the maximum allowed size of an input data set associated with the property
     prpNumOverloads : UINT32;                                      //< the number of overloads associated with this property name
     prpHistoryDepthShort : UINT16;                                 //< the depth of the short term history kept for this property
     prpHistoryDepthLong  : UINT16;                                 //< the depth of the long term history kept for this property
     prpFormat       : BYTE;                                        //< the default output data format accepted for this property
     prpFormatIn     : BYTE;                                        //< is the default input data format accepted for this property
     prpAccess       : BYTE;                                        //< the allowed data access for this property (either CA_READ, CA_WRITE or both)
     prpGraphType    : BYTE;                                        //< is the preferred graph type for output data associated with this property, e.g. GT_LIN, GT_LOG, etc
     rngUnits        : array[0..UNITS_SIZE-1] of char;              //< a string containing the natural units of the x-axis (range) in case of a spectrum type array
     rngMinValue     : float;                                       //< the lower limit of x-axis range associated with the property
     rngMaxValue     : float;                                       //< the upper limit of x-axis range associated with the property
     numRows         : UINT16;                                      //< the number of rows in an array property
     rowSize         : UINT16;                                      //< the size of a row in an array property
     prpArrayType    : UINT16;                                      //< the property array type (AT_NONE, AT_SCALAR, AT_CHANNEL, AT_SPECTRUM, etc.)
     reserved        : array[0..2] of UINT16;                       //< reserved field
 end;
 pPrpQueryStruct = ^PrpQueryStruct;

 XPropertyQueryStruct = record
     prpName         : array[0..PROPERTY_NAME_SHORTSIZE-1] of char; //< the property name
     prpDescription  : array[0..63] of char;                        //< the property description
     prpRedirection  : array[0..STATUS_SHORTSIZE-1] of char;        //< a redirection string if the property is redirected to another server
     prpTag          : array[0..TAG_NAME_SHORTSIZE-1] of char;      //< the data tag for output data sets (CF_STRUCT, CF_BITFIELD, CF_HISTORY, etc.)
     prpTagIn        : array[0..TAG_NAME_SHORTSIZE-1] of char;      //< the data tag for input data sets (CF_STRUCT, CF_BITFIELD, CF_HISTORY, etc.)
     prpUnits        : array[0..15] of char;                        //< a string containing the natural units of the data returned by the property
     prpMinValue     : float;                                       //< the lower limit of valid data associated with the property
     prpMaxValue     : float;                                       //< the upper limit of valid data associated with the property
     prpSize         : UINT32;                                      //< the maximum allowed size of an output data set associated with the property
     prpSizeIn       : UINT32;                                      //< the maximum allowed size of an input data set associated with the property
     prpNumOverloads : UINT32;                                      //< the number of overloads associated with this property name
     prpHistoryDepthShort : UINT16;                                 //< the depth of the short term history kept for this property
     prpHistoryDepthLong  : UINT16;                                 //< the depth of the long term history kept for this property
     prpFormat       : BYTE;                                        //< the default output data format accepted for this property
     prpFormatIn     : BYTE;                                        //< is the default input data format accepted for this property
     prpAccess       : BYTE;                                        //< the allowed data access for this property (either CA_READ, CA_WRITE or both)
     prpGraphType    : BYTE;                                        //< is the preferred graph type for output data associated with this property, e.g. GT_LIN, GT_LOG, etc
     rngUnits        : array[0..15] of char;                        //< a string containing the natural units of the x-axis (range) in case of a spectrum type array
     rngMinValue     : float;                                       //< the lower limit of x-axis range associated with the property
     rngMaxValue     : float;                                       //< the upper limit of x-axis range associated with the property
     numRows         : UINT16;                                      //< the number of rows in an array property
     rowSize         : UINT16;                                      //< the size of a row in an array property
     prpArrayType    : UINT16;                                      //< the property array type (AT_NONE, AT_SCALAR, AT_CHANNEL, AT_SPECTRUM, etc.)
     reserved        : array[0..2] of UINT16;                       //< reserved field
 end;
 pXPropertyQueryStruct = ^XPropertyQueryStruct;

 ConTblInfo = record
     expName         : array[0..EXPORT_NAME_SIZE-1] of char;        //< eqm function name
     prpName         : array[0..PROPERTY_NAME_SIZE-1] of char;      //< eqm property
     devName         : array[0..DEVICE_NAME_SIZE-1] of char;        //< eqm device name
     context         : array[0..CONTEXT_NAME_SIZE-1] of char;       //< eqm context
     eqm             : array[0..EQM_NAME_SIZE-1] of char;           //<
     mode            : UINT16;                                      //< used by subscriptions
     access          : short;                                       //< data access
     formatIn        : short;                                       //< eqp data format down
     formatOut       : short;                                       //< eqp data format up
     sizeIn          : UINT32;                                      //< data array size down
     sizeOut         : UINT32;                                      //< data array size up
     strTagIn        : array[0..TAG_NAME_SIZE-1] of char;           //< eqm structure tag down
     strTagOut       : array[0..TAG_NAME_SIZE-1] of char;           //< eqm structure tag up
     pollRate        : UINT32;                                      //< polling rate for connectioned clients
     heartbeat       : UINT32;                                      //< used by subscriptions
     starttime       : UINT32;                                      //< entry into table (used by subscriptions)
     dtimestamp      : UINT32;                                      //< arriving TINE timestamp
     dtimestampMSEC  : int;                                         //< arriving TINE timestamp (msec fraction)
     counter         : int;                                         //< current subscription counter
     linkStatus      : short;                                       //< current link status
     reserved        : short;                                       //< reserved for future
 end;
 pConTblInfo = ^ConTblInfo;

implementation

end.
